Imports TsChanManLibrary

Public Class dlgRun

    Private _cn As ChannelBase
    Private bServerResponse As Boolean = False
    Private iStatus As Integer = 0
    Private sServerResponse As String = ""

    Private AxVLC As AxAXVLC.AxVLCPlugin ' VLC ActiveX Plugin


    Sub New(ByVal cn As ChannelBase)
        InitializeComponent()

        _cn = cn
    End Sub

    Private Sub dlgRun_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        ' By removing the handler, the socket is still connected asynchronysly, but
        ' discards it, it the server sends data back to this client
        RemoveHandler cSockets.OnDataReceived, AddressOf OnReceiveData

        If Not AxVLC Is Nothing Then AxVLC.Dispose()
    End Sub

    Private Sub dlgRun_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        Me.Text += _cn.sName & "..." ' Add channel name to window title
        pb_Logo.Image = _cn.bLogo

        Try
            cSockets.SendChannelDataToServer(_cn) ' Send data for the selected channel to the server
        Catch ex As Exception
            MessageBox.Show("Could not send channel data to server." & vbCrLf & vbCrLf & _
                            "The error reported was: " & vbCrLf & ex.Message, _
                            "Error while trying to establish connection to server", MessageBoxButtons.OK, _
                            MessageBoxIcon.Error)
            Me.DialogResult = Windows.Forms.DialogResult.Cancel
            Return
        End Try

        ' As the client waits for data to come back from the server asynchronysly, 
        ' the event 'cSockets.OnDataReceived' get's called in this case
        AddHandler cSockets.OnDataReceived, AddressOf OnReceiveData

        TimerStatus.Start() ' Start a timer, so we can update the ProgressBar
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click

        If Not AxVLC Is Nothing Then
            cSockets.StopChannelOnServer()
        End If

        Me.Close()
    End Sub

    ''' <summary>
    ''' Gets called via the event 'cSockets.OnDataReceived' after the server sends 
    ''' data back to the client
    ''' </summary>
    ''' <param name="text"></param>
    ''' <remarks></remarks>
    Private Sub OnReceiveData(ByVal text As String)

        ' As this method is executed in another thread, we don't access UI elements
        ' The Timer will do this for us...
        bServerResponse = True
        sServerResponse = text
    End Sub

    Private Sub TimerStatus_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TimerStatus.Tick

        ' The 'bServerResponse' is set from outside via 'OnReceiveData()' -- asynchronysly
        If bServerResponse Then
            TimerStatus.Stop()
            Try
                If CheckServerResponse() Then Me.DialogResult = Windows.Forms.DialogResult.OK
            Catch ex As Exception
                MessageBox.Show("Received response from server, but could not process the response." & _
                                Environment.NewLine & Environment.NewLine & "Reason:" & _
                                Environment.NewLine & ex.Message, _
                                "Error reading response", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Me.DialogResult = Windows.Forms.DialogResult.Cancel
            End Try
        Else ' Let the progess bar display some faked values so the user see's the application is still working
            If iStatus <= 100 Then
                ProgressStart.Value = iStatus
                iStatus += 1
            Else ' It shouldn't take that long, means something went wrong
                TimerStatus.Stop()

                MessageBox.Show("Did not receive a response from the server within a valid period of time.", _
                                "Timeout while waiting for the response", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

                Me.DialogResult = Windows.Forms.DialogResult.Cancel
            End If
        End If
    End Sub

    ''' <summary>
    ''' Evaluate the Response received from the server (server started/server not started)
    ''' </summary>
    ''' <remarks></remarks>
    Private Function CheckServerResponse() As Boolean
        If sServerResponse.Contains("|") Then
            If sServerResponse.Split("|")(0) = "1" Then ' 1 means everything went right

                Dim bInternalPlayer As Boolean = cSettings.Cfg.bInternalVlcPlayer

                Dim sVlcIpPort As String = "http://" & cSettings.Cfg.sServerIP & ":"
                Dim IndexOfURL As Integer = cSettings.Cfg.sVlcServerCommand.IndexOf("url={0}") + 8 ' + 8 da url={0} nicht bentigt
                If IndexOfURL <> 7 Then ' 7 = -1 + 8

                    ' To get the portnumber, beginn enumerating the port number until no
                    ' digit is remaining
                    While (Char.IsDigit(cSettings.Cfg.sVlcServerCommand, IndexOfURL))
                        sVlcIpPort += cSettings.Cfg.sVlcServerCommand.Substring(IndexOfURL, 1)
                        IndexOfURL += 1
                    End While


                    If cSettings.Cfg.bInternalVlcPlayer Then
                        Try
                            AxVLC = New AxAXVLC.AxVLCPlugin()
                            AxVLC.Dock = DockStyle.Fill

                            For Each c As Control In PanelOben.Controls
                                c.Visible = False
                            Next

                            PanelOben.Controls.Add(AxVLC)
                            AxVLC.addTarget(sVlcIpPort, Type.Missing, Global.AXVLC.VLCPlaylistMode.VLCPlayListInsertAndGo, 0)

                            Me.Text = "Running channel " & _cn.sName & "..."
                            Me.Size = New Size(735, 650) ' TODO: scale 100% to PAL video! (and check aspect ratio!)
                            Me.Location = New Point(100, 45)

                            btnCancel.Text = "Stop channel"
                            btnCancel.Location = ProgressStart.Location ' Place the Stop-Button where the ProgressBar was
                            btnCancel.Image = My.Resources.Resources.computer_delete

                            ProgressStart.Visible = False

                            Return False ' Means "Don't close that window!"
                        Catch ex As Exception
                            bInternalPlayer = False
                            AxVLC.Dispose() ' Make sure no other exception is thrown
                        End Try
                    End If

                    If Not bInternalPlayer Then
                        Dim p As New Process
                        p.StartInfo.FileName = cSettings.Cfg.sVlcExe

                        ' After the server told us that VLC has been started it might take a while 
                        ' until VLC actually starts streaming (eg. CPU-usage was 100% on the server)
                        p.StartInfo.Arguments = sVlcIpPort & " --ipv4-timeout=30000"
                        p.Start()
                        Return True
                    End If
                Else
                    MessageBox.Show("Could not parse VLC Server command")
                End If

            Else
                MessageBox.Show("The server was unable to start TSReader or VLC." & vbCrLf & _
                                vbCrLf & "The error reported was:" & vbCrLf & _
                                sServerResponse.Split("|")(1), "Server failure", _
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return True
            End If
        Else
            MessageBox.Show("Received unknown response from server: " + Environment.NewLine + _
                            sServerResponse, "Server error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return True
        End If
    End Function
End Class
