Imports TsChanManLibrary

Public Class frmRecord

    ''' <summary>
    ''' The channel to record
    ''' </summary>
    ''' <remarks></remarks>
    Private cn As ChannelBase

    Public rConfig As RecordingSettings

    ''' <summary>
    ''' Main constructor
    ''' </summary>
    ''' <param name="_channel">channel to be edited</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal _channel As ChannelBase)

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        cn = _channel
    End Sub


    Private Sub pb_OK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles pb_OK.Click

        If ValidateInput() Then
            Dim fs As Integer = 0 ' FileSize for splitting
            Dim dt As DateTime = DateTime.MinValue ' Time to start the recording

            If cb_Split.Checked Then fs = Integer.Parse(df_FileSize.Text)

            If cbTimedRecording.Checked AndAlso Not DateTime.TryParse(mtbDate.Text + " " + mtbTime.Text, dt) Then
                MessageBox.Show("Could not process DateTime value: " + Environment.NewLine _
                                + mtbDate.Text + " " + mtbTime.Text)
                Return
            End If

            If cbTimedRecording.Checked Then
                MessageBox.Show("TSReader Channel Manager will now create a new scheduled task to run at the time you specified" & _
                                Environment.NewLine & Environment.NewLine & _
                                "You might also need to enter your user account password to create this task.", _
                                "Scheduled task notice", MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If

            rConfig = New RecordingSettings(df_File.Text, TimeSpan.Parse(mtb_RecordTime.Text), _
                                            cb_Split.Checked, fs, _
                                            cbTimedRecording.Checked, _
                                            dt, cb_PStream.Checked, _
                                            cbIncludeSubtitle.Checked, _
                                            cbEntireTS.Checked)

            Me.DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

    Private Sub frmRecord_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Zuletzt verwendete Datei setzen
        df_File.Text = cSettings.Cfg.sLastRecordingFile

        ' Kanalname und Info als berschrift
        lbl_RecordChannelName.Text = "Record channel (" & frm_MainWindow.lsv_ChannelList.SelectedItems(0).Text & ")"
    End Sub

    ''' <summary>
    ''' Button for the user to select where to put the recording file
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub pb_File_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles pb_File.Click
        Dim saveFileDialog1 As New SaveFileDialog()
        saveFileDialog1.InitialDirectory = "C:\"
        saveFileDialog1.Filter = "Transport-Stream (*.ts)|*.ts"
        saveFileDialog1.FilterIndex = 1
        saveFileDialog1.RestoreDirectory = True

        If saveFileDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            df_File.Text = saveFileDialog1.FileName
        End If
    End Sub

    Private Function ValidateInput() As Boolean
        Dim bReturn As Boolean = True

        If cbTimedRecording.Checked Then
            If Not Date.TryParse(mtbDate.Text, Nothing) Then
                ErrorProviderRecord.SetError(mtbDate, "Date not in the correct format")
                bReturn = False
            Else
                ErrorProviderRecord.SetError(mtbDate, String.Empty)
            End If

            If Not TimeSpan.TryParse(mtbTime.Text, Nothing) Then
                ErrorProviderRecord.SetError(mtbTime, "Time not in the correct format")
                bReturn = False
            Else
                ErrorProviderRecord.SetError(mtbTime, String.Empty)
            End If
        End If

        If cb_Split.Checked Then
            If Not Integer.TryParse(df_FileSize.Text, Nothing) Then
                ErrorProviderRecord.SetError(df_FileSize, "Please specify a valid split size")
                bReturn = False
            Else
                ErrorProviderRecord.SetError(df_FileSize, String.Empty)
            End If
        End If

        If String.IsNullOrEmpty(df_File.Text) Then
            ErrorProviderRecord.SetError(df_File, "Please provide a filename")
            bReturn = False
        Else
            ErrorProviderRecord.SetError(df_File, String.Empty)
        End If

        If Not TimeSpan.TryParse(mtb_RecordTime.Text, Nothing) Then
            ErrorProviderRecord.SetError(mtb_RecordTime, "Time not in the correct format")
            bReturn = False
        Else
            ErrorProviderRecord.SetError(mtb_RecordTime, String.Empty)
        End If

        Return bReturn
    End Function

    ''' <summary>
    ''' Checkbox for 'split recording'
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cb_Split_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cb_Split.CheckedChanged
        If cb_Split.Checked Then
            PanelSplitFile.Enabled = True
        Else
            PanelSplitFile.Enabled = False
        End If
    End Sub

    ''' <summary>
    ''' Checkbox for 'timed recording'
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cbTimedRecording_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbTimedRecording.CheckedChanged
        If cbTimedRecording.Checked Then
            PanelTimedRecord.Enabled = True
        Else
            PanelTimedRecord.Enabled = False
        End If
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
    End Sub
End Class